#include <memory> // auto_ptr
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <stdlib.h> // getenv()
#include "url_gzip.hpp"
#include "pstreams.hpp"
#include <s11n.net/s11n/s11n_debuggering_macros.hpp>
#include <s11n.net/s11n/plugin/path_finder.hpp>
#include <s11n.net/s11n/plugin/plugin_config.hpp> // s11n_CONFIG_DEFAULT_BINS_PATH

#include <s11n.net/s11n/classload.hpp>
#include <s11n.net/s11n/export.hpp> // unfortunate kludge

namespace s11n { namespace io { namespace pstreams {

	using ::s11n::io::url_parser;

	gzip_stream_factory::gzip_stream_factory() : ::s11n::io::url_stream_factory()
	{
	}


	gzip_stream_factory::~gzip_stream_factory()
	{
	}

	std::ostream *
	gzip_stream_factory::do_get_ostream( url_parser const & url ) const
	{
		if( ! url.good() ) return 0;
		std::string cmd = "gzip -c > ";
		char quote = '\'';
		std::string path = url.host(); // kludge: the filename may be parsed as a host name.
		if( path.empty() )
		{
			path = url.path();
		}
		if( std::string::npos != path.find('\'') )
		{
			quote = '"';
		}
 		cmd += quote;
 		cmd += path;
 		cmd += quote;
		std::ostream * os = new redi::opstream( cmd );
		if( ! os->good() )
		{
			delete os;
			os = 0;
		}
		return os;
	}

	std::istream *
	gzip_stream_factory::do_get_istream( url_parser const & url ) const
	{
		if( ! url.good() ) return 0;
		std::string cmd = "gzip -d < ";
		std::string path = url.host(); // kludge: the filename may be parsed as a host name.
		if( path.empty() )
		{
			path = url.path();
		}
		char quote = '\'';
		if( std::string::npos != path.find('\'') )
		{
			quote = '"';
		}
		cmd += quote;
		cmd += path;
		cmd += quote;
		std::istream * is = new redi::ipstream( cmd );
		if( ! is->good() )
		{
			delete is;
			is = 0;
		}
		return is;
	}

	void pstreams_gzip_static_init()
	{
		/******
		       Register with the classloader and set up some classloader aliases...
		       only if we find the gzip app.
		******/
		::s11n::plugin::path_finder p;
		char * path = ::getenv("PATH");
		p.path( path ? std::string(path) : s11n_CONFIG_DEFAULT_BINS_PATH );
#if defined(WIN32)
		p.extensions( ".exe" );
#endif
		std::string file = p.find( "gzip" );
		if( file.empty() )
		{
			CERR << "gzip not found in $PATH. gzip support disabled.\n";
			return;
		}
		typedef s11n::fac::factory_mgr<s11n::io::url_stream_factory> FacT;
		s11n::fac::register_subtype<s11n::io::url_stream_factory, gzip_stream_factory>( "gzip" );
		FacT::instance().alias( "gunzip", "gzip" );
	}

	bool pstreams_gzip_static_init_placeholder = (pstreams_gzip_static_init(),true);

}}} // namespaces
