#include <memory> // auto_ptr
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <stdlib.h> // getenv()
#include "url_uu.hpp"
#include "pstreams.hpp"
#include <s11n.net/s11n/s11n_debuggering_macros.hpp>
#include <s11n.net/s11n/plugin/path_finder.hpp>

#include <s11n.net/s11n/classload.hpp>
#include <s11n.net/s11n/export.hpp> // unfortunate kludge

#define UU_STREAM_FACTORY_CLASS_NAME "s11n::io::pstreams::uu_stream_factory"
namespace s11n { namespace io { namespace pstreams {

	using ::s11n::io::url_parser;

	uu_stream_factory::uu_stream_factory() : ::s11n::io::url_stream_factory()
	{
	}


	uu_stream_factory::~uu_stream_factory()
	{
	}

	std::ostream *
	uu_stream_factory::do_get_ostream( url_parser const & url ) const
	{
		if( ! url.good() ) return 0;
		std::string cmd = "uuencode ";
		char quote = '\'';
		std::string path = url.host(); // kludge: the filename may be parsed as a host name.
		if( path.empty() )
		{
			path = url.path();
		}
		if( std::string::npos != path.find('\'') )
		{
			quote = '"';
		}
		cmd += quote;
		cmd += path;
		cmd += quote;
 		cmd += " > ";
 		cmd += quote;
 		cmd += path;
 		cmd += quote;
		// CERR << "uu: ostream cmd="<<cmd<<'\n';
		std::ostream * os = new redi::opstream( cmd );
		if( ! os->good() )
		{
			delete os;
			os = 0;
		}
		return os;
	}

	std::istream *
	uu_stream_factory::do_get_istream( url_parser const & url ) const
	{
		if( ! url.good() ) return 0;
		std::string cmd = "uudecode -o- ";
		std::string path = url.host(); // kludge: the filename may be parsed as a host name.
		if( path.empty() )
		{
			path = url.path();
		}
		char quote = '\'';
		if( std::string::npos != path.find('\'') )
		{
			quote = '"';
		}
		cmd += quote;
		cmd += path;
		cmd += quote;
		std::istream * is = new redi::ipstream( cmd );
		if( ! is->good() )
		{
			delete is;
			is = 0;
		}
		return is;
	}

	void pstreams_uu_static_init()
	{
		/******
		       Set up some classloader aliases...
		 ******/
		typedef s11n::fac::factory_mgr<url_stream_factory> FacT;
		FacT & fac = FacT::instance();

		::s11n::plugin::path_finder p;
		char * path = ::getenv("PATH");
		p.path( path ? path : "/bin:/usr/bin:/usr/local/bin" );
#if defined(WIN32)
		p.extensions( ".exe" );
#endif
		std::string file = p.find( "uuencode" );
		if( ! file.empty() )
		{
			fac.alias( "uu", UU_STREAM_FACTORY_CLASS_NAME );
		}
		else
		{
			CERR << "uuencode not found in $PATH. uuencode/decode support disabled.\n";
		}
	}

	bool pstreams_uu_static_init_placeholder = (pstreams_uu_static_init(),true);

}}} // namespaces

/************************************************************************
Now we register the various handlers...
************************************************************************/

#define S11N_FACREG_TYPE s11n::io::pstreams::uu_stream_factory
#define S11N_FACREG_INTERFACE_TYPE s11n::io::url_stream_factory
#define S11N_FACREG_TYPE_NAME UU_STREAM_FACTORY_CLASS_NAME
#include <s11n.net/s11n/factory_reg.hpp>
#undef UU_STREAM_FACTORY_CLASS_NAME

